@@ -1,10 +1,11 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Ryan Dellenbaugh
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
+from quodlibet import _
 from quodlibet.plugins.query import QueryPlugin, QueryPluginError
 from quodlibet.query._parser import QueryParser
 
