@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2012 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -21,6 +20,9 @@ class MonkeysAudioFile(APEv2File):
         super(MonkeysAudioFile, self).__init__(filename, audio)
         self["~#length"] = int(audio.info.length)
         self["~#channels"] = audio.info.channels
+        self["~#samplerate"] = audio.info.sample_rate
+        if hasattr(audio.info, "bits_per_sample"):
+            self["~#bitdepth"] = audio.info.bits_per_sample
         self.sanitize(filename)
 
 loader = MonkeysAudioFile
