@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman
 #
 # This program is free software; you can redistribute it and/or modify
@@ -8,8 +7,6 @@
 
 from senf import fsnative, path2fsn
 
-from quodlibet.compat import text_type, PY3
-
 from ._audio import AudioFile
 
 
@@ -23,9 +20,8 @@ class RemoteFile(AudioFile):
     format = "Remote File"
 
     def __init__(self, uri):
-        if PY3:
-            assert not isinstance(uri, bytes)
-        self["~uri"] = text_type(uri)
+        assert not isinstance(uri, bytes)
+        self["~uri"] = str(uri)
         self.sanitize(fsnative(self["~uri"]))
 
     def __getitem__(self, key):
