@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2011 Joe Wreschnig, Michael Urman, Steven Robertson,
 #           2011-2014 Christoph Reiter
 #
@@ -28,7 +27,6 @@ from quodlibet.util import fver, sanitize_tags, MainRunner, MainRunnerError, \
 from quodlibet.player import PlayerError
 from quodlibet.player._base import BasePlayer
 from quodlibet.qltk.notif import Task
-from quodlibet.compat import iteritems
 from quodlibet.formats.mod import ModFile
 
 from .util import (parse_gstreamer_taglist, TagListWrapper, iter_to_list,
@@ -154,7 +152,13 @@ class BufferingWrapper(object):
 
 
 def sink_has_external_state(sink):
-    return sink.get_factory().get_name() == "pulsesink"
+    sink_name = sink.get_factory().get_name()
+
+    if sink_name == "wasapisink":
+        # https://bugzilla.gnome.org/show_bug.cgi?id=796386
+        return hasattr(sink.props, "volume")
+    else:
+        return sink_name == "pulsesink"
 
 
 def sink_state_is_valid(sink):
@@ -519,7 +523,7 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
 
         if not self.has_external_volume:
             # Restore volume/ReplayGain and mute state
-            self.volume = self._volume
+            self.props.volume = self._volume
             self.mute = self._mute
 
         # ReplayGain information gets lost when destroying
@@ -960,12 +964,12 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         tags = TagListWrapper(tags, merge=True)
         tags = parse_gstreamer_taglist(tags)
 
-        for key, value in iteritems(sanitize_tags(tags, stream=False)):
+        for key, value in sanitize_tags(tags, stream=False).items():
             if self.song.get(key) != value:
                 changed = True
                 self.song[key] = value
 
-        for key, value in iteritems(sanitize_tags(tags, stream=True)):
+        for key, value in sanitize_tags(tags, stream=True).items():
             if new_info.get(key) != value:
                 info_changed = True
                 new_info[key] = value
