@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2007-2008 Joe Wreschnig
 #           2009,2010 Steven Robertson
 #           2009-2013 Christoph Reiter
@@ -13,7 +12,6 @@ from gi.repository import GObject
 from quodlibet.formats import AudioFile
 from quodlibet.util import print_d
 from quodlibet import config
-from quodlibet.compat import listfilter
 
 
 class Equalizer(object):
@@ -125,7 +123,7 @@ class BasePlayer(GObject.GObject, Equalizer):
         """
 
         if self.song and config.getboolean("player", "replaygain"):
-            profiles = listfilter(None, self.replaygain_profiles)[0]
+            profiles = list(filter(None, self.replaygain_profiles))[0]
             fb_gain = config.getfloat("player", "fallback_gain")
             pa_gain = config.getfloat("player", "pre_amp_gain")
             scale = self.song.replay_gain(profiles, pa_gain, fb_gain)
@@ -153,11 +151,12 @@ class BasePlayer(GObject.GObject, Equalizer):
 
     @property
     def volume(self):
-        return self.props.volume
+        """Use a cubic scale for the externally exposed volume"""
+        return self.props.volume ** (1.0 / 3.0)
 
     @volume.setter
     def volume(self, v):
-        self.props.volume = min(1.0, max(0.0, v))
+        self.props.volume = min(1.0, max(0.0, v ** 3.0))
 
     @property
     def mute(self):
