@@ -1,30 +1,32 @@
-# -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig, Michael Urman
-#           2013 Nick Boultbee
+#           2013, 2015 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import Gtk, Gdk
 
-from quodlibet.qltk.entry import UndoEntry
+from quodlibet import _
 from quodlibet.qltk.window import Dialog
+from quodlibet.qltk import Icons
+from quodlibet.qltk.entry import UndoEntry
 
 
 class GetStringDialog(Dialog):
     """Simple dialog to return a string from the user"""
-    _WIDTH = 300
 
-    def __init__(self, parent, title, text, okbutton=Gtk.STOCK_OPEN):
+    def __init__(self, parent, title, text,
+                 button_label=_("_OK"), button_icon=Icons.DOCUMENT_OPEN,
+                 tooltip=None):
         super(GetStringDialog, self).__init__(
             title=title, transient_for=parent, use_header_bar=True)
 
         self.set_border_width(6)
-        self.set_default_size(width=self._WIDTH, height=0)
         self.set_resizable(True)
-        self.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                         okbutton, Gtk.ResponseType.OK)
+        self.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
+        self.add_icon_button(button_label, button_icon, Gtk.ResponseType.OK)
         self.vbox.set_spacing(6)
         self.set_default_response(Gtk.ResponseType.OK)
 
@@ -36,6 +38,9 @@ class GetStringDialog(Dialog):
         box.pack_start(lab, True, True, 0)
 
         self._val = UndoEntry()
+        if tooltip:
+            self._val.set_tooltip_text(tooltip)
+        self._val.set_max_width_chars(50)
         box.pack_start(self._val, True, True, 0)
 
         self.vbox.pack_start(box, True, True, 0)
